@@ -21,6 +21,8 @@
 
 #include "gnome-settings-bus.h"
 
+#include "gsd-resources.h"
+
 #ifndef PLUGIN_NAME
 #error Include PLUGIN_CFLAGS in the daemon s CFLAGS
 #endif /* !PLUGIN_NAME */
@@ -219,6 +221,9 @@ main (int argc, char **argv)
         textdomain (GETTEXT_PACKAGE);
         setlocale (LC_ALL, "");
 
+	/* Ensure we don't lose resources during linkage */
+	g_resources_register (gsd_get_resource ());
+
         set_empty_gtk_theme (TRUE);
 
 #ifdef GDK_BACKEND
